import asyncio
import heapq
import time
from typing import NamedTuple, List, Optional

from rich.console import Console
from rich.control import Control

from .screen import ScreenPoint


class Render:
    def __init__(self, console: Console = None):
        self.console = console or Console()

    @property
    def screen_size(self):
        return self.console.size

    def move_cursor_home(self):
        self.console.control(Control.home())

    def clear(self):
        self.console.control(Control.clear())

    def draw_string_at(self, p: ScreenPoint, s: str):
        def clamp(minimum, v, maximum):
            return max(minimum, min(v, maximum))

        # if p.x > self.screen_size.width - 1:
        #     return
        # if p.y > self.screen_size.height - 2:
        #     return
        # if p.x < 0:
        #     return
        # if p.y < 0:
        #     return
        # x = int(p.x)
        # y = int(p.y)

        x = int(clamp(0, p.x, self.screen_size.width - 1))
        y = int(clamp(0, p.y, self.screen_size.height - 2))

        self.console.control(Control.move_to(x, y))
        self.console.out(s, end='')


class AnimationRender(Render):

    class TimedDrawStruct(NamedTuple):
        time_s: float
        point: ScreenPoint
        char: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # This will be maintained sorted (priority queue)
        self._scheduled: List[AnimationRender.TimedDrawStruct] = []
        self._evt_stop = asyncio.Event()

    def schedule_draw(
            self,
            timeout: float, p: ScreenPoint, s: str,
            clean_after: Optional[float] = None
    ):
        heapq.heappush(self._scheduled, AnimationRender.TimedDrawStruct(timeout, p, s))

        if clean_after is not None:
            self.schedule_draw(timeout + clean_after, p, ' ', None)

    async def render_frames(self):
        _start_time = time.monotonic()

        while self._scheduled:
            st = heapq.heappop(self._scheduled)
            remaining_s = st.time_s - (time.monotonic() - _start_time)

            if remaining_s > 0:
                await asyncio.sleep(remaining_s)
            else:
                self.draw_string_at(st.point, st.char)



import math
from abc import ABC, abstractmethod
from typing import List

from .screen import ScreenPoint, ScreenPointDrawing


class Brush(ABC):
    """
    Brush base class

    You may implement your own brush style.
    """

    @abstractmethod
    def get_points(self, x, y, angle) -> List[ScreenPointDrawing]:
        """Return all the brush points"""
        pass


class BrushWipe(Brush):
    width = 6
    deformation_factor = 2

    def get_points(self, x, y, angle) -> List[ScreenPointDrawing]:
        """Return all the brush points"""

        half_width = self.width / 2
        opposite_angle = angle + math.pi

        ret = []
        for step in range(0, int(half_width * self.deformation_factor)):
            fac = half_width / (half_width * self.deformation_factor) * step
            p = ScreenPoint(
                x + math.cos(angle)          * fac * self.deformation_factor,
                y + math.sin(angle)          * fac
            )
            ret.append(ScreenPointDrawing(p, '#'))

            p = ScreenPoint(
                x + math.cos(opposite_angle) * fac * self.deformation_factor,
                y + math.sin(opposite_angle) * fac
            )
            ret.append(ScreenPointDrawing(p, '#'))

        return ret



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wipe_clean']

package_data = \
{'': ['*']}

install_requires = \
['rich>=11.2.0,<12.0.0']

entry_points = \
{'console_scripts': ['wipe-clean = wipe_clean.main:cli']}

setup_kwargs = {
    'name': 'wipe-clean',
    'version': '0.1.4',
    'description': 'Clear your terminal in a ritual way',
    'long_description': "# Wipe Clean\n\n![made-with-python](https://img.shields.io/badge/Made%20with-Python-1f425f.svg)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/wipe-clean?logo=python)\n\n[![PyPI](https://img.shields.io/pypi/v/wipe-clean?logo=pypi)](https://pypi.org/project/wipe-clean/)\n[![PyPI - Wheel](https://img.shields.io/pypi/wheel/wipe-clean)](https://pypi.org/project/wipe-clean/)\n[![PyPI - Status](https://img.shields.io/pypi/status/wipe-clean)](https://pypi.org/project/wipe-clean/)\n\n[![Linux](https://img.shields.io/badge/Linux-FCC624?logo=linux&logoColor=black)](https://github.com/wenoptics/python-wipe-clean)\n[![Mac OS](https://img.shields.io/badge/MacOS--9cf?logo=Apple&style=social)](https://github.com/wenoptics/python-wipe-clean)\n[![Windows](https://img.shields.io/badge/Windows-0078D6?logo=windows&logoColor=white)](https://github.com/wenoptics/python-wipe-clean)\n\n---\n\nClear your terminal in a ritual way. Works on Windows, Linux and macOS.\n\n```bash\npip install wipe-clean\n```\n\n![demo](https://github.com/wenoptics/python-wipe-clean/blob/master/doc/terminal.gif?raw=true)\n\n\n> `wipe-clean` requires Python 3.6.1 and above. Note that Python 3.6.0 is\n not supported due to lack of `NamedTuples` typing.\n\n> `wipe-clean` is moving to pursue **0-dependency**, see the [Roadmap](#roadmap).\n\n\n## Usages\n\nJust:\n\n```bash\nwipe-clean\n```\n\n## Advanced Usages\n\n### 1. Use API\n\n```python\nfrom wipe_clean.main import cli as wc_cli\n\nwc_cli()\n```\n\n### 2. Customization\n\nIt's possible to design your own brush shape and animation!\n\n#### Example Brush\n\nTo create a new brush type, just implement the `Brush` interface.\n\n```python\nfrom wipe_clean.brush import Brush, ScreenPointDrawing, ScreenPoint as P\n\nclass Wipe2x2(Brush):\n    def get_points(self, x, y, angle) -> List[ScreenPointDrawing]:\n        return [\n            ScreenPointDrawing(P(x    , y    ), '#'),\n            ScreenPointDrawing(P(x + 1, y    ), '#'),\n            ScreenPointDrawing(P(x    , y + 1), '#'),\n            ScreenPointDrawing(P(x + 1, y + 1), '#'),\n        ]\n```\n\nThis will define a brush like this:\n\n```text\n##\n##\n```\n\n#### Example Path\n\nSimilarly, you can implement the `Path` interface.\n\n```python\nimport math\nfrom wipe_clean.path import Path, PathPoint, ScreenPoint as P\n\nclass MySimplePath(Path):\n    def get_points(self) -> Iterable[PathPoint]:\n        return [\n            PathPoint(P(10, 10), math.radians(45)),\n            PathPoint(P(20, 5), math.radians(0)),\n            PathPoint(P(40, 20), math.radians(90)),\n        ]\n```\n\n\n## Roadmap\n\nSee [`DEVELOPMENT.md`](./DEVELOPMENT.md)\n\n\n## Related Projects\n\n- [`JeanJouliaCode/wipeclean`](https://github.com/JeanJouliaCode/wipeClean) - JavaScript version\n\n  _The first brush type (`BrushWipe`) and path animations (`PathZigZag`, `PathRectEdge`) are direct ports\n  of `JeanJouliaCode/wipeclean`. Credits go to JeanJouliaCode!_\n\n- [`Textualize/rich`](https://github.com/Textualize/rich) - _Inspiring textual UI library_\n",
    'author': 'wenoptk',
    'author_email': 'wenoptics@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/wenoptics/python-wipe-clean',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)
